unit RegMru;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus,Registry;

type
  pMenuEntry = ^tMenuEntry;

  TMenuEntry = record
    Caption: string[80];
    Index: integer;
    item: TMenuItem;
  end;

  pMenuInfo = ^TMenuInfo;

  TMenuInfo = record
    Caption: string[80];
    Enabled: Boolean;
    Checked: Boolean;
    Tag: longint;
    Index: integer;
  end;


  TRootKey = (HKeyClassesRoot, HKeyCurrentUser, HKeyLocalMachine, HKeyUsers,
              HKeyPerformanceData, HKeyCurrentConfig, HKeyDynData);

  TNotifyItemClick = procedure(Sender: TObject; MenuItem: TMenuItem;
                               const FileName: String; var CanDelete: Boolean)of object;

  TOnGetItems = procedure(Sender: TObject; item: TMenuItem; index: integer)of object;

  TRegMru = class(TComponent)
  private
    Flist: TList;
    FMenuLine: TMenuItem;
    FAdded: Boolean;
    bAdd: Boolean;
    function GetCurrentPath: string;
    function GetCurrentKey: HKEY;
    function GetLazyWrite: Boolean;
    procedure SetLazyWrite(newValue: Boolean);
  protected
    FMenu: TMenuItem;
    FMax: integer;
    FNumber: Boolean;
    FLine: Boolean;
    FRegistry: TRegistry;
    FRootKey: TRootKey;
    FCompanyName: string;
    FApplicationName: string;
    FVersion: string;
    FOwnerName: string;
    FSection: string;
    FSerialize: Boolean;
    FLazyWrite: Boolean;
    FNotifyItemClick: TNotifyItemClick;
    FOnGetItems: TOnGetItems;
    function CreateRegistryPath: string;
    procedure MenuItemClick(Sender: TObject);
    procedure Remove(item: TMenuItem);
    procedure NumberEntries;
    procedure SetSerialize(newVal: Boolean);
    procedure SetRootKey(Key: TRootKey);
    procedure CleanRegistry;
    procedure ReArrange(firstItem: TMenuItem);
    procedure BeforeDestruction; override;
    function AddItem(const Title: string;Checked, Enabled: Boolean;Tag: integer;
                     Reorder: Boolean; Index: integer = 0): TMenuItem;
    function CheckEqualNames(Name: string): Boolean;
    procedure Loaded; override;
    procedure SetMaxItems(newValue: integer);
    function GetActiveRegistry: Boolean;
    procedure SetActiveRegistry(newVal: Boolean);
  public
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
    procedure GetAllMenuItems;
    function Add(const Title: string;Checked, Enabled: Boolean; Tag: integer; Index: integer = 0): TMenuItem;
    procedure RemoveMenuItem(FileName: string);
    procedure ReadSavedItems;
    procedure SaveMenuItems;
    function GetItemIndex(item: TMenuItem): integer;
    function CreateKey(KeyName: string): Boolean;
    function DeleteKey(KeyName: string): Boolean;
    function GetDataInfo(ValueName: string; var Info: TRegDataInfo): Boolean;
    function GetDataSize(ValueName: string): integer;
    function GetDataType(ValueName: string): TRegDataType;
    function GetKeyInfo(var Value: TRegKeyInfo): Boolean;
    procedure GetKeyNames(Strings: TStrings);
    procedure GetValueNames(Strings: TStrings);
    function HasSubKeys: Boolean;
    function KeyExists(Key: string): Boolean;
    function LoadKey(const Key,FileName: string): Boolean;
    procedure MoveKey(const OldName,NewName: string; Delete: Boolean);
    function OpenKey(KeyName: string; CanCreate: Boolean): Boolean;
    function OpenKeyReadOnly(KeyName: string): Boolean;
    function ReadBinaryData(const Name: string; var Buffer; BufSize: integer): integer;
    function ReadBool(const Name: string): Boolean;
    function ReadCurrency(const Name: string): Currency;
    function ReadDate(const Name: string): TDateTime;
    function ReadDateTime(const Name: string): TDateTime;
    function ReadFloat(const Name: string): double;
    function ReadInteger(const Name: string): integer;
    function ReadString(const Name: string): string;
    function ReadTime(const Name: string): TDateTime;
    function RegistryConnect(const UNCName: string): Boolean;
    procedure RenameValue(const OldName,NewName: string);
    function ReplaceKey(const Key,FileName,BackupFileName: string): Boolean;
    function RestoreKey(const Key,FileName: string): Boolean;
    function SaveKey(const Key,FileName: string): Boolean;
    function UnloadKey(const Key: string): Boolean;
    function ValueExists(const Name: string): Boolean;
    procedure WriteBinaryData(const Name: string; var Buffer; BufSize: integer);
    procedure WriteBool(const Name: string; Value: Boolean);
    procedure WriteCurrency(const Name: string; Value: Currency);
    procedure WriteDate(const Name: string; Value: TDateTime);
    procedure WriteDateTime(const Name: string; Value: TDateTime);
    procedure WriteExpandString(const Name: string; Value: string);
    procedure WriteFloat(const Name: string; Value: double);
    procedure WriteInteger(const Name: string; Value: integer);
    procedure WriteString(const Name: string; Value: string);
    procedure WriteTime(const Name: string; Value: TDateTime);
    procedure CloseKey;
  published
    property MaxItems: integer read FMax write SetMaxItems;
    property LazyWrite: Boolean read GetLazyWrite write SetLazyWrite;
    property CurrentPath : string read GetCurrentPath;
    property CurrentKey: HKEY read GetCurrentKey;
    property ApplicationName: string read FApplicationName write FApplicationName;
    property CompanyName: string read FCompanyName write FCompanyName;
    property RootKey: TRootKey read FRootKey write SetRootKey default HkeyCurrentUser;
    property OwnerName: string read FOwnerName write FOwnerName;
    property Section: string read FSection write FSection;
    property ApplicationVersion: string read FVersion write FVersion;
    property SerializeList: Boolean read FSerialize write SetSerialize;
    property ActiveRegistry: Boolean read GetActiveRegistry write SetActiveRegistry;
    property Separator: Boolean read FLine write FLine;
    property NumberItems: Boolean read FNumber write FNumber;
    property MenuItem:  TMenuItem read FMenu write FMenu;
    property OnItemClick: TNotifyItemClick read FNotifyItemClick write FNotifyItemClick;
    property OnGetMenuItems: TOnGetItems read FOnGetItems write FOnGetItems;
  end;

procedure Register;

implementation

var
  Roots: array[TRootKey] of HKey = (HKEY_CLASSES_ROOT, HKEY_CURRENT_USER, HKEY_LOCAL_MACHINE,
                                    HKEY_USERS, HKEY_PERFORMANCE_DATA, HKEY_CURRENT_CONFIG,
                                    HKEY_DYN_DATA);

procedure Register;
begin
  RegisterComponents('Samples', [TRegMru]);
end;

constructor TRegMru.Create (AOwner: TComponent);
begin
  inherited Create(AOwner);
  FMax:= -1;
  FNumber:= TRUE;
  FLine:= TRUE;
  FMenuLine:= nil;
  FMenu:= nil;
  FAdded:= FALSE;
  FList:= TList.Create;
  FRegistry := nil;
  FNotifyItemClick := nil;
  FOnGetItems := nil;
  FRootKey := HKeyCurrentUser;
  FLazyWrite := TRUE;
  bAdd:=TRUE;
  FOwnerName := TComponent(Owner).Name;
  FSection := 'MRUHistory';
end;

destructor TRegMru.Destroy;
var i: integer;
begin
  if FRegistry <> nil then FRegistry.Free;
  for i:=0 to FList.Count-1 do Dispose(pMenuEntry(FList[i]));
  FList.Free;
  inherited Destroy;
end;

procedure TRegMru.Loaded;
var bActivity: Boolean;
begin
  if(not(csDesigning in ComponentState))and(SerializeList)then
  begin
    bActivity := ActiveRegistry;
    if not bActivity then ActiveRegistry := TRUE;
    ReadSavedItems;
    NumberEntries;
    if not bActivity then ActiveRegistry := FALSE;
  end;
end;

function TRegMru.GetActiveRegistry: Boolean;
begin
  Result := FRegistry <> nil;
end;

procedure TRegMru.SetActiveRegistry(newVal: Boolean);
begin
  if newVal then
  begin
    if FRegistry = nil then
    begin
      FRegistry := TRegistry.Create;
      FRegistry.RootKey := Roots[FRootKey];
      FRegistry.LazyWrite := FLazyWrite;
    end;
  end
  else
  begin
    if FRegistry <> nil then
    begin
      FRegistry.Free;
      FRegistry := nil;
    end;
  end;
end;

procedure TRegMru.SetRootKey(Key: TRootKey);
begin
  FRootKey:=Key;
  if FRegistry <> nil then FRegistry.RootKey:=Roots[FRootKey];
end;

function TRegMru.CreateRegistryPath: string;
begin
  Result:='\Software\'+FCompanyName+'\'+FApplicationName+'\'+FVersion+
  '\' + FOwnerName + '\' + FSection;
end;

procedure TRegMru.CleanRegistry;
var List: TStringList;
    i: integer;
begin
  if FRegistry.OpenKey(CreateRegistryPath,FALSE) then
  begin
    List:=TStringList.Create;
    FRegistry.GetValueNames(List);
    if List.Count > 0 then
    begin
      for i := 0 to  List.Count -1 do
      if(UpperCase(List.Strings[i]) <> 'DEFAULT')then
        FRegistry.DeleteValue(List.Strings[i]);
    end;
    List.Free;
    FRegistry.CloseKey;
  end;
end;

procedure TRegMru.ReadSavedItems;
var List: TStringList;
    i,nCount,j,len,nDBIndex: integer;
    s,s1: string;
    bActivity: Boolean;
begin
  bActivity := ActiveRegistry;
  if not ActiveRegistry then ActiveRegistry := TRUE;
  if FRegistry.OpenKey(CreateRegistryPath,FALSE) then
  begin
    List:=TStringList.Create;
    FRegistry.GetValueNames(List);
    if List.Count > 0 then
    begin
      nCount := 0;
      j := 0;
      for i := 0 to  List.Count -1 do
      begin
        if(UpperCase(List.Strings[i]) = 'COUNT')then
        begin
          s:=FRegistry.ReadString(List.Strings[i]);
          nCount := StrToIntDef(s,0);
          inc(j);
        end;
        if(UpperCase(List.Strings[i]) = 'MAXFILES')then
        begin
          s:=FRegistry.ReadString(List.Strings[i]);
          FMax := StrToIntDef(s,0);
          inc(j);
        end;
        if j = 2 then break;
      end;
      if nCount > 0 then
      begin
        for i:=0 to List.Count-1 do
          if(UpperCase(List.Strings[i])<>'COUNT')and(UpperCase(List.Strings[i])<>'DEFAULT')
             and(Uppercase(List.Strings[i])<>'MAXFILES') then
          begin
            s:=FRegistry.ReadString(List.Strings[i]);
            j := LastDelimiter('|',s);
            if(j <> 0)then
            begin
              len := Length(s);
              s1:=Copy(s,j+1,len);
              nDBIndex := StrToIntDef(s1,0);
              s1 := Copy(s,1,j - 1);
              s:=s1;
            end
            else nDBIndex := 0;
            if(not CheckEqualNames(s))then AddItem(s,FALSE,TRUE,i,FALSE,nDBIndex);
          end;
      end;
    end;
    List.Free;
    FRegistry.CloseKey;
  end;
  if not bActivity then ActiveRegistry := FALSE;
end;

procedure TRegMru.SaveMenuItems;
var p: pMenuEntry;
    i: integer;
    s,value,key: string;
    bActivity: Boolean;
begin
  bActivity := ActiveRegistry;
  if not ActiveRegistry then ActiveRegistry := TRUE;
  CleanRegistry;
  if FRegistry.OpenKey(CreateRegistryPath,TRUE)then
  begin
    value:=IntToStr(FList.Count);
    FRegistry.WriteString('Count',value);
    value:=IntToStr(FMax);
    FRegistry.WriteString('MaxFiles',value);
    for i:=0 to FList.Count-1 do
    begin
      p:=FList.Items[i];
      s:=p^.Caption;
      if p^.Index <>0 then s:=s+'|'+IntToStr(p^.Index);
      key:='H'+IntToStr(i);
      FRegistry.WriteString(key,s);
    end;
    FRegistry.CloseKey;
  end;
  if not bActivity then ActiveRegistry := FALSE;
end;

procedure TRegMru.MenuItemClick(Sender: TObject);
var bDelete: Boolean;
    pItem: TMenuItem;
    i,n: integer;
    s: string;
begin
  bDelete:=FALSE;
  pItem:=TMenuItem(Sender);
  n:=FList.Count;
  dec(n);
  for i:=0 to n do
    if(pItem = pMenuEntry(FList.Items[i])^.item)then break;
  if Assigned(FNotifyItemClick)then
  begin
    s:=pMenuEntry(FList.Items[i])^.Caption;
    bAdd:=FALSE; //set false to avoid double rearange
    FNotifyItemClick(Self,TMenuItem(Sender),s,bDelete);
    bAdd:=TRUE;
  end;
  if bDelete then Remove(TMenuItem(Sender))
  else ReArrange(TMenuItem(Sender));
end;

function TRegMru.CheckEqualNames(Name: string): Boolean;
var i: integer;
begin
  Result:=FALSE;
  for i:=0 to FList.Count - 1 do
   if Uppercase(Name) = Uppercase(pMenuEntry(FList.Items[i]).Caption) then
   begin
     ReArrange(pMenuEntry(FList.Items[i])^.item);
     Result:=TRUE;
     break;
   end;
end;

function TRegMru.Add(const Title: string;Checked, Enabled: Boolean; Tag: integer; Index: integer): TMenuItem;
var i,n: integer;
    p: pMenuEntry;
begin
  Result:=nil;
  if bAdd then
  begin
    if(not CheckEqualNames(Title))then
    begin
      if(FMax > -1)and(FList.Count >= FMax)then
      begin
        n:=FList.Count - FMax +1;
        //for i:=0 to n-1 do Remove(pMenuEntry(FList.Items[i])^.item);
        for i:= FList.Count-1 downto FList.Count - n do Remove(pMenuEntry(FList.Items[i])^.item);
      end;
      Result:=AddItem(Title,Checked,Enabled,Tag,TRUE,Index);
    end
    else for i:=0 to FList.Count -1 do
           begin
             p:=FList.Items[i];
             if p^.Caption = Title then
             begin
               p^.Index := Index;
               Result := p^.item;
               break;
             end;
           end;
  end;
end;

function TRegMru.GetItemIndex(item: TMenuItem): integer;
var i: integer;
    p: pMenuEntry;
begin
  Result := 0;
  for i:=0 to FList.Count - 1 do
  begin
    p:=FList.Items[i];
    if(p<>nil)then
      if(p^.item = item)then
      begin
        Result := p^.Index;
        break;
      end;
  end;
end;

function TRegMru.AddItem(const Title: string;Checked,Enabled: Boolean;
                      Tag: integer; Reorder: Boolean; Index: integer): TMenuItem;
var p: pMenuEntry;
begin
  Result:=nil;
  if(FMax > -1)and(FList.Count >= FMax)then Exit;
  if(not FAdded) and FLine then
  begin
    FMenuLine:= NewLine;
    FMenu.Add (FMenuline);
    FAdded:= TRUE;
  end;
  New (p);
  if p <> nil then
  begin
    p^.Caption:= title;
    p^.item:= NewItem (Title,0,Checked,Enabled,MenuItemClick,0,'');
    p^.item.Tag:= tag;
    p^.Index:=Index;
    Result:= p^.item;
    FList.Add (p);
    FMenu.Add (p^.item);
    if Reorder then ReArrange(p^.item);
  end;
end;

procedure TRegMru.Remove (item: TMenuItem);
var p: pMenuEntry;
    i: INTEGER;
begin
  for i:= 0 to FList.Count - 1 do
  begin
    p:= FList.Items[i];
    if p <> nil then
    begin
      if p^.item = item then
      begin
        FMenu.Remove (p^.item);
        Dispose (p);
        FList.Delete (i);
        FList.Pack;
        NumberEntries;
        if(FList.Count <= 0) and Assigned(FMenuLine)then
        begin
          FMenu.Remove (FMenuLine);
          FMenuLine:=nil;
          FAdded:= FALSE;
        end;
        break;
      end;
    end;
  end;
end;

procedure TRegMru.RemoveMenuItem(FileName: string);
var i: integer;
begin
  for i:=0 to FList.Count - 1 do
  begin
    if UpperCase(pMenuEntry(FList.Items[i])^.Caption) = UpperCase(FileName)then
    begin
      Remove(pMenuEntry(FList.Items[i])^.item);
      break;
    end;
  end;
end;

procedure TRegMru.NumberEntries;
var m: pMenuEntry;
    s: string;
    i: integer;
begin
  if FNumber then
  begin
    for i:= 0 to FList.Count - 1 do
    begin
      m:= FList.Items[i];
      if m <> nil then
      begin
        if i < 8 then s:= '&' + IntToStr (i+1) + ' ' + m^.Caption
        else s:= IntToStr (i+1) + ' ' + m^.Caption;
        m.item.Caption:= s;
      end;
    end;
  end;
end;

procedure TRegMru.SetSerialize(newVal: Boolean);
begin
   FSerialize:=newVal;
end;

procedure TRegMru.BeforeDestruction;
begin
  if(FSerialize)then
  begin
    if not ActiveRegistry then ActiveRegistry := TRUE;
    SaveMenuItems;
  end;
end;

procedure TRegMru.ReArrange(firstItem: TMenuItem);
var i,j: integer;
    pInfo: pMenuInfo;
    BufList: TList;
begin
  BufList:=TList.Create;
  for i:=0 to FList.Count do
  begin
    New(pInfo);
    FillChar(pInfo^,sizeof(TMenuInfo),0);
    BufList.Add(pInfo);
    pInfo := nil;
  end;
  j:=1;
  for i:=0 to FList.Count - 1 do
  begin
    if pMenuEntry(FList.Items[i])^.item = firstItem then
    begin
      pMenuInfo(BufList.Items[0])^.Caption:=pMenuEntry(FList.Items[i])^.Caption;
      pMenuInfo(BufList.Items[0])^.Checked:=pMenuEntry(FList.Items[i])^.item.Checked;
      pMenuInfo(BufList.Items[0])^.Enabled:=pMenuEntry(FList.Items[i])^.item.Enabled;
      pMenuInfo(BufList.Items[0])^.Tag:=pMenuEntry(FList.Items[i])^.item.Tag;
      pMenuInfo(BufList.Items[0])^.Index:=pMenuEntry(FList.Items[i])^.Index;
    end
    else
    begin
      pMenuInfo(BufList.Items[j])^.Caption:=pMenuEntry(FList.Items[i])^.Caption;
      pMenuInfo(BufList.Items[j])^.Checked:=pMenuEntry(FList.Items[i])^.item.Checked;
      pMenuInfo(BufList.Items[j])^.Enabled:=pMenuEntry(FList.Items[i])^.item.Enabled;
      pMenuInfo(BufList.Items[j])^.Tag:=pMenuEntry(FList.Items[i])^.item.Tag;
      pMenuInfo(BufList.Items[j])^.Index:=pMenuEntry(FList.Items[i])^.Index;
      j:=j+1;
    end;
  end;
  for i:=0 to FList.Count - 1 do
  begin
    pMenuEntry(FList.Items[i])^.Caption:=pMenuInfo(BufList.Items[i])^.Caption;
    pMenuEntry(FList.Items[i])^.item.Caption:=pMenuInfo(BufList.Items[i])^.Caption;
    pMenuEntry(FList.Items[i])^.item.Checked:=pMenuInfo(BufList.Items[i])^.Checked;
    pMenuEntry(FList.Items[i])^.item.Enabled:=pMenuInfo(BufList.Items[i])^.Enabled;
    pMenuEntry(FList.Items[i])^.item.Tag:=pMenuInfo(BufList.Items[i])^.Tag;
    pMenuEntry(FList.Items[i])^.Index:=pMenuInfo(BufList.Items[i])^.Index;
  end;
  for i:=0 to FList.Count - 1 do Dispose(pMenuInfo(BufList.Items[i]));
  BufList.Free;
  NumberEntries;
end;

procedure TRegMru.SetMaxItems(newValue: integer);
var m,n,i: integer;
    p: pMenuEntry;
begin
  FMax := newValue;
  if FMax > -1 then
  begin
    n := FList.Count;
    if FMax < n then
    begin
      m := n - FMax;
      i := 1;
      while m > 0 do
      begin
        p := pMenuEntry(FList.Items[n-i]);
        Remove(p^.item);
        dec(m);
        inc(i);
      end;
    end;
  end;  
end;

function TRegMru.GetCurrentPath: string;
begin
  if ActiveRegistry then Result:=FRegistry.CurrentPath
  else Result := '';
end;

function TRegMru.GetCurrentKey: HKEY;
begin
  if ActiveRegistry then Result:=FRegistry.CurrentKey
  else Result := 0;
end;

function TRegMru.GetLazyWrite: Boolean;
begin
  Result := FLazyWrite;
end;

procedure TRegMru.SetLazyWrite(newValue: Boolean);
begin
  FLazyWrite := newValue;
  if FRegistry <> nil then FRegistry.LazyWrite:=newValue;
end;

function TRegMru.CreateKey(KeyName: string): Boolean;
begin
  if ActiveRegistry then Result:=FRegistry.CreateKey(KeyName)
  else Result := FALSE;
end;

function TRegMru.DeleteKey(KeyName: string): Boolean;
begin
  if ActiveRegistry then Result:=FRegistry.DeleteKey(KeyName)
  else Result := FALSE;
end;

function TRegMru.GetDataInfo(ValueName: string; var Info: TRegDataInfo): Boolean;
begin
  if ActiveRegistry then Result:=FRegistry.GetDataInfo(ValueName,Info)
  else Result := FALSE;
end;

function TRegMru.GetDataSize(ValueName: string): integer;
begin
  if ActiveRegistry then Result:=FRegistry.GetDataSize(ValueName)
  else Result := 0;
end;

function TRegMru.GetDataType(ValueName: string): TRegDataType;
begin
  if ActiveRegistry then Result:=FRegistry.GetDataType(ValueName)
  else Result := rdUnknown;
end;

function TRegMru.GetKeyInfo(var Value: TRegKeyInfo): Boolean;
begin
  if ActiveRegistry then Result:=FRegistry.GetKeyInfo(Value)
  else Result := FALSE;
end;

procedure TRegMru.GetKeyNames(Strings: TStrings);
begin
  if ActiveRegistry then FRegistry.GetKeyNames(Strings)
  else Strings.Clear;
end;

procedure TRegMru.GetValueNames(Strings: TStrings);
begin
  if ActiveRegistry then FRegistry.GetValueNames(Strings)
  else Strings.Clear;
end;

function TRegMru.HasSubKeys: Boolean;
begin
  if ActiveRegistry then Result:=FRegistry.HasSubKeys
  else Result := FALSE;
end;

function TRegMru.KeyExists(Key: string): Boolean;
begin
  if ActiveRegistry then Result:=FRegistry.KeyExists(Key)
  else Result := FALSE;
end;

function TRegMru.LoadKey(const Key,FileName: string): Boolean;
begin
  if ActiveRegistry then Result:=FRegistry.LoadKey(Key,FileName)
  else Result := FALSE;
end;

procedure TRegMru.MoveKey(const OldName,NewName: string; Delete: Boolean);
begin
  if ActiveRegistry then FRegistry.MoveKey(OldName,NewName,Delete);
end;

function TRegMru.OpenKey(KeyName: string; CanCreate: Boolean): Boolean;
begin
  if ActiveRegistry then Result:=FRegistry.OpenKey(KeyName,CanCreate)
  else Result := FALSE;
end;

function TRegMru.OpenKeyReadOnly(KeyName: string): Boolean;
begin
  if ActiveRegistry then Result:=FRegistry.OpenKeyReadOnly(KeyName)
  else Result := FALSE;
end;

function TRegMru.ReadBinaryData(const Name: string; var Buffer; BufSize: integer): integer;
begin
  if ActiveRegistry then Result:=FRegistry.ReadBinaryData(Name,Buffer,BufSize)
  else Result := 0;
end;

function TRegMru.ReadBool(const Name: string): Boolean;
begin
  if ActiveRegistry then Result:=FRegistry.ReadBool(Name)
  else Result := FALSE;
end;

function TRegMru.ReadCurrency(const Name: string): Currency;
begin
  if ActiveRegistry then Result:=FRegistry.ReadCurrency(Name)
  else Result := 0;
end;

function TRegMru.ReadDate(const Name: string): TDateTime;
begin
  if ActiveRegistry then Result:=FRegistry.ReadDate(Name)
  else Result := 0;
end;

function TRegMru.ReadDateTime(const Name: string): TDateTime;
begin
  if ActiveRegistry then Result:=FRegistry.ReadDateTime(Name)
  else Result := 0;
end;

function TRegMru.ReadFloat(const Name: string): double;
begin
  if ActiveRegistry then Result:=FRegistry.ReadFloat(Name)
  else Result := 0;
end;

function TRegMru.ReadInteger(const Name: string): integer;
begin
  if ActiveRegistry then Result:=FRegistry.ReadInteger(Name)
  else Result := 0;
end;

function TRegMru.ReadString(const Name: string): string;
begin
  if ActiveRegistry then Result:=FRegistry.ReadString(Name)
  else Result := '';
end;

function TRegMru.ReadTime(const Name: string): TDateTime;
begin
  if ActiveRegistry then Result:=FRegistry.ReadTime(Name)
  else Result := 0;
end;

function TRegMru.RegistryConnect(const UNCName: string): Boolean;
begin
  if ActiveRegistry then Result:=FRegistry.RegistryConnect(UNCName)
  else Result := FALSE;
end;

procedure TRegMru.RenameValue(const OldName,NewName: string);
begin
  if ActiveRegistry then FRegistry.RenameValue(OldName,NewName);
end;

function TRegMru.ReplaceKey(const Key,FileName,BackupFileName: string): Boolean;
begin
  if ActiveRegistry then Result:=FRegistry.ReplaceKey(Key,FileName,BackupFileName)
  else Result := FALSE;
end;

function TRegMru.RestoreKey(const Key,FileName: string): Boolean;
begin
  if ActiveRegistry then Result:=FRegistry.RestoreKey(Key,FileName)
  else Result := FALSE;
end;

function TRegMru.SaveKey(const Key,FileName: string): Boolean;
begin
  if ActiveRegistry then Result:=FRegistry.SaveKey(Key,FileName)
  else Result := FALSE;
end;


function TRegMru.UnloadKey(const Key: string): Boolean;
begin
  if ActiveRegistry then Result:=FRegistry.UnLoadKey(Key)
  else Result := FALSE;
end;

function TRegMru.ValueExists(const Name: string): Boolean;
begin
  if ActiveRegistry then Result:=FRegistry.ValueExists(Name)
  else Result := FALSE;
end;

procedure TRegMru.WriteBinaryData(const Name: string; var Buffer; BufSize: integer);
begin
  if ActiveRegistry then FRegistry.WriteBinaryData(Name,Buffer,BufSize);
end;

procedure TRegMru.WriteBool(const Name: string; Value: Boolean);
begin
  if ActiveRegistry then FRegistry.WriteBool(Name,Value);
end;

procedure TRegMru.WriteCurrency(const Name: string; Value: Currency);
begin
  if ActiveRegistry then FRegistry.WriteCurrency(Name,Value);
end;

procedure TRegMru.WriteDate(const Name: string; Value: TDateTime);
begin
  if ActiveRegistry then FRegistry.WriteDate(Name,Value);
end;

procedure TRegMru.WriteDateTime(const Name: string; Value: TDateTime);
begin
  if ActiveRegistry then FRegistry.WriteDateTime(Name,Value);
end;

procedure TRegMru.WriteExpandString(const Name: string; Value: string);
begin
  if ActiveRegistry then FRegistry.WriteExpandString(Name,Value);
end;

procedure TRegMru.WriteFloat(const Name: string; Value: double);
begin
  if ActiveRegistry then FRegistry.WriteFloat(Name,Value);
end;

procedure TRegMru.WriteInteger(const Name: string; Value: integer);
begin
  if ActiveRegistry then FRegistry.WriteInteger(Name,Value);
end;

procedure TRegMru.WriteString(const Name: string; Value: string);
begin
  if ActiveRegistry then FRegistry.WriteString(Name,Value);
end;

procedure TRegMru.WriteTime(const Name: string; Value: TDateTime);
begin
  if ActiveRegistry then FRegistry.WriteTime(Name,Value);
end;

procedure TRegMru.CloseKey;
begin
  if ActiveRegistry then FRegistry.CloseKey;
end;

procedure TRegMru.GetAllMenuItems;
var i: integer;
begin
  if(FList.Count > 0)and(Assigned(FOnGetItems))then
    for i:=0 to FList.Count - 1 do FOnGetItems(Self,pMenuEntry(FList.Items[i])^.item,i);
end;

end.
